@@ -14,34 +14,52 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library.
 #
+"""
+Access logs in known locations to find information about them.
+"""
 
 import os
 import re
-import sys
-import string
 import codecs
 import platform
 
-py3 = platform.python_version()[0] == '3'
-if py3:
-    unicode = str
+PY3 = platform.python_version()[0] == '3'
+
+if PY3:
+     # pylint: disable=W0622
+     unicode = str
+     basestring = str
 
 from dateutil import parser as dateparse
 
 MATCHER = r'(?P<date>\w+ +\d+ +\d\d:\d\d:\d\d) (?P<host>\w+) ' + \
         r'CRON\[(?P<pid>\d+)\]: \((?P<user>\w+)\) CMD \((?P<cmd>.*)\)'
 
-def size(filename):
-    return os.stat(filename)[6]
-
 class LogReader(object):
     """Opens a Log file, reading backwards and watching for changes"""
     def __init__(self, filename, mass=4096):
         self.filename = filename
-        self.pipe     = codecs.open(filename, 'r', encoding='utf-8')
-        self.mass     = mass
-        self.size     = -1
-        self.read     = -1
+        self.mass = mass
+        self.size = -1
+        self.read = -1
+        self.pipe = None
+
+    def __enter__(self):
+        self.size = os.stat(self.filename)[6]
+        self.pipe = codecs.open(self.filename, 'r', encoding='utf-8')
+        return self
+
+    def __exit__(self, error_type, value, traceback):
+        self.pipe.close()
+
+    def __iter__(self):
+        if self.pipe is None:
+            with self as reader:
+                for (offset, line) in reader.readlines():
+                    yield line
+        else:
+            for (offset, line) in self.readlines():
+                yield line
 
     def readlines(self, until=0):
         """Iterator for reading lines from a file backwards"""
@@ -49,10 +67,8 @@ class LogReader(object):
             raise IOError("Can't readline, no opened file.")
         # Always seek to the end of the file, this accounts for file updates
         # that happen during our running process.
-        self.size = size(self.filename)
-        block_num = 0
-        location  = self.size
-        halfline  = ''
+        location = self.size
+        halfline = ''
 
         while location > until:
             location -= self.mass
@@ -73,23 +89,22 @@ class LogReader(object):
                 yield (loc, line)
                 loc -= len(line)
 
-    def __iter__(self):
-        for (offset, line) in self.readlines():
-            yield line
 
 
 class CronLog(LogReader):
+    """Use the LogReader to make a Cron specific log reader"""
     def __init__(self, filename='/var/log/syslog', user=None):
         LogReader.__init__(self, filename)
         self.user = user
 
     def for_program(self, command):
+        """Return log entries for this specific command name"""
         return ProgramLog(self, command)
 
     def __iter__(self):
-        for (offset, line) in self.readlines():
-            c = re.match(MATCHER, unicode(line))
-            datum = c and c.groupdict()
+        for line in super(CronLog, self).__iter__():
+            match = re.match(MATCHER, unicode(line))
+            datum = match and match.groupdict()
             if datum and (not self.user or datum['user'] == self.user):
                 datum['date'] = dateparse.parse(datum['date'])
                 yield datum
