@@ -1,5 +1,5 @@
 #
-# Copyright 2017, Martin Owens <doctormo@gmail.com>
+# Copyright 2021, Martin Owens <doctormo@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -107,7 +107,7 @@ except ImportError:
                           " install ordereddict 1.1 from pypi for python2.6")
 
 __pkgname__ = 'python-crontab'
-__version__ = '2.5.1'
+__version__ = '2.6.0'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^\n]*?)(\s+#\s*([^\n]*)|$)')
@@ -154,7 +154,7 @@ ZERO_PAD = False
 
 LOG = logging.getLogger('crontab')
 
-CRONCMD = "/usr/bin/crontab"
+CRON_COMMAND = "/usr/bin/crontab"
 SHELL = os.environ.get('SHELL', '/bin/sh')
 # The shell won't actually work on windows here, but
 # it should be updated later in the below conditional.
@@ -198,7 +198,7 @@ def _unicode(text):
     return text
 
 
-class CronTab(object):
+class CronTab:
     """
     Crontab object which can access any time based cron using the standard.
 
@@ -211,12 +211,12 @@ class CronTab(object):
     tab     - Use a string variable as the crontab instead of installed crontab
     tabfile - Use a file for the crontab instead of installed crontab
     log     - Filename for logfile instead of /var/log/syslog
-
     """
     def __init__(self, user=None, tab=None, tabfile=None, log=None):
         self.lines = None
         self.crons = None
         self.filen = None
+        self.cron_command = CRON_COMMAND
         self.env = None
         self._parked_env = OrderedDict()
         # Protect windows users
@@ -237,7 +237,7 @@ class CronTab(object):
     @property
     def log(self):
         """Returns the CronLog object for this tab (user or root tab only)"""
-        from cronlog import CronLog
+        from cronlog import CronLog # pylint: disable=import-outside-toplevel
         if self._log is None or isinstance(self._log, basestring):
             self._log = CronLog(self._log, user=self.user or 'root')
         return self._log
@@ -288,7 +288,7 @@ class CronTab(object):
                 lines = fhl.readlines()
 
         elif self.user:
-            (out, err) = open_pipe(CRONCMD, l='', **self.user_opt).communicate()
+            (out, err) = open_pipe(self.cron_command, l='', **self.user_opt).communicate()
             if err and 'no crontab for' in unicode(err):
                 pass
             elif err:
@@ -354,11 +354,11 @@ class CronTab(object):
                 os.unlink(path)
                 raise IOError("Please specify user or filename to write.")
 
-            proc = open_pipe(CRONCMD, path, **self.user_opt)
+            proc = open_pipe(self.cron_command, path, **self.user_opt)
             ret = proc.wait()
             if ret != 0:
                 raise IOError("Program Error: {} returned {}: {}".format(
-                    CRONCMD, ret, proc.stderr.read()))
+                    self.cron_command, ret, proc.stderr.read()))
             proc.stdout.close()
             proc.stderr.close()
             os.unlink(path)
@@ -387,10 +387,14 @@ class CronTab(object):
             sleep(kwargs.get('cadence', 60))
             count += 1
 
-    def render(self, errors=False):
+    def render(self, errors=False, specials=True):
         """Render this crontab as it would be in the crontab.
 
         errors - Should we not comment out invalid entries and cause errors?
+        specials - Turn known times into keywords such as "@daily"
+            True - (default) force all values to be converted (unless SYSTEMV)
+            False - force all values back from being a keyword
+            None - don't change the special keyword use
         """
         crons = []
         for line in self.lines:
@@ -404,7 +408,7 @@ class CronTab(object):
             elif isinstance(line, CronItem):
                 if not line.is_valid() and not errors:
                     line.enabled = False
-                crons.append(unicode(line).strip())
+                crons.append(line.render(specials=specials).strip())
 
         # Environment variables are attached to cron lines so order will
         # always work no matter how you add lines in the middle of the stack.
@@ -421,7 +425,8 @@ class CronTab(object):
         """
         if not user and self.user is False:
             raise ValueError("User is required for system crontabs.")
-        item = CronItem(command, comment, user=user, cron=self, pre_comment=pre_comment)
+        item = CronItem(command, comment, user=user, pre_comment=pre_comment)
+        item.cron = self
         self.append(item)
         return item
 
@@ -479,9 +484,9 @@ class CronTab(object):
             raise AttributeError("Invalid use: remove_all(command='cmd')")
         if 'command' in kwargs:
             return self.remove(*self.find_command(kwargs['command']))
-        elif 'comment' in kwargs:
+        if 'comment' in kwargs:
             return self.remove(*self.find_comment(kwargs['comment']))
-        elif 'time' in kwargs:
+        if 'time' in kwargs:
             return self.remove(*self.find_time(kwargs['time']))
         return self.remove(*self.crons[:])
 
@@ -509,10 +514,9 @@ class CronTab(object):
                 sibling.env.update(env)
                 sibling.env.job = sibling
                 break
-            elif sibling == '':
-                self.lines.remove(sibling)
-            else:
+            if sibling != '':
                 break
+            self.lines.remove(sibling)
 
         self.crons.remove(item)
         self.lines.remove(item)
@@ -522,9 +526,9 @@ class CronTab(object):
         kind = 'System ' if self._user is False else ''
         if self.filen:
             return "<%sCronTab '%s'>" % (kind, self.filen)
-        elif self.user and not self.user_opt:
+        if self.user and not self.user_opt:
             return "<My CronTab>"
-        elif self.user:
+        if self.user:
             return "<User CronTab '%s'>" % self.user
         return "<Unattached %sCronTab>" % kind
 
@@ -546,13 +550,13 @@ class CronTab(object):
         return self.render()
 
 
-class CronItem(object):
+class CronItem:
     """
     An item which objectifies a single line of a crontab and
     May be considered to be a cron job object.
     """
-    def __init__(self, command='', comment='', user=None, cron=None, pre_comment=False):
-        self.cron = cron
+    def __init__(self, command='', comment='', user=None, pre_comment=False):
+        self.cron = None
         self.user = user
         self.valid = False
         self.enabled = True
@@ -575,7 +579,6 @@ class CronItem(object):
 
         if command:
             self.set_command(command)
-            self.valid = True
 
     def __hash__(self):
         return hash((self.command, self.comment, self.hour, self.minute, self.dow))
@@ -588,7 +591,8 @@ class CronItem(object):
     @classmethod
     def from_line(cls, line, user=None, cron=None):
         """Generate CronItem from a cron-line and parse out command and comment"""
-        obj = cls(user=user, cron=cron)
+        obj = cls(user=user)
+        obj.cron = cron
         obj.parse(line.strip())
         return obj
 
@@ -596,8 +600,7 @@ class CronItem(object):
         """Delete this item and remove it from it's parent"""
         if not self.cron:
             raise UnboundLocalError("Cron item is not in a crontab!")
-        else:
-            self.cron.remove(self)
+        self.cron.remove(self)
 
     def set_command(self, cmd, parse_stdin=False):
         """Set the command and filter as needed"""
@@ -606,6 +609,7 @@ class CronItem(object):
             if '\n' in cmd:
                 cmd, self.stdin = cmd.split('\n', 1)
         self.command = _unicode(cmd.strip())
+        self.valid = True
 
     def set_comment(self, cmt, pre_comment=False):
         """Set the comment and don't filter, pre_comment indicates comment appears
@@ -648,7 +652,6 @@ class CronItem(object):
             self.set_command(result[0][-3], True)
         try:
             self.setall(*result[0][:-3])
-            self.valid = True
         except (ValueError, KeyError) as err:
             if self.enabled:
                 LOG.error(str(err))
@@ -669,15 +672,18 @@ class CronItem(object):
         """Return true if this job is valid"""
         return self.valid
 
-    def render(self):
+    def render(self, specials=True):
         """Render this set cron-job to a string"""
+        if not self.is_valid() and self.enabled:
+            raise ValueError('Refusing to render invalid crontab.'
+                             ' Disable to continue.')
         command = _unicode(self.command).replace(u'%', u'\\%')
         user = ''
         if self.cron and self.cron.user is False:
             if not self.user:
                 raise ValueError("Job to system-cron format, no user set!")
             user = self.user + ' '
-        result = u"%s %s%s" % (unicode(self.slices), user, command)
+        result = u"%s %s%s" % (self.slices.render(specials=specials), user, command)
         if self.stdin:
             result += ' %' + self.stdin.replace('\n', '%')
         if not self.enabled:
@@ -776,7 +782,7 @@ class CronItem(object):
             date_from = datetime.now()
         try:
             # Croniter is an optional import
-            from croniter.croniter import croniter
+            from croniter.croniter import croniter # pylint: disable=import-outside-toplevel
         except ImportError:
             raise ImportError("Croniter not available. Please install croniter"
                               " python module via pip or your package manager")
@@ -789,7 +795,7 @@ class CronItem(object):
         **kw - Keyword arguments to pass to cron_descriptor (see docs)
         """
         try:
-            from cron_descriptor import ExpressionDescriptor
+            from cron_descriptor import ExpressionDescriptor # pylint: disable=import-outside-toplevel
         except ImportError:
             raise ImportError("cron_descriptor not available. Please install"\
               "cron_descriptor python module via pip or your package manager")
@@ -868,13 +874,10 @@ class CronItem(object):
         return self.__unicode__()
 
     def __unicode__(self):
-        if not self.is_valid() and self.enabled:
-            raise ValueError('Refusing to render invalid crontab.'
-                             ' Disable to continue.')
         return self.render()
 
 
-class Every(object):
+class Every:
     """Provide an interface to the job.every() method:
         Available Calls:
           minute, minutes, hour, hours, dom, doms, month, months, dow, dows
@@ -950,15 +953,15 @@ class CronSlices(list):
             return CronSlices._parse_str(value)
         if isinstance(value, CronItem):
             return value.slices, None
-        elif isinstance(value, datetime):
+        if isinstance(value, datetime):
             return [value.minute, value.hour, value.day, value.month, '*'], None
-        elif isinstance(value, time):
+        if isinstance(value, time):
             return [value.minute, value.hour, '*', '*', '*'], None
-        elif isinstance(value, date):
+        if isinstance(value, date):
             return [0, 0, value.day, value.month, '*'], None
             # It might be possible to later understand timedelta objects
             # but there's no convincing mathematics to do the conversion yet.
-        elif not isinstance(value, (list, tuple)):
+        if not isinstance(value, (list, tuple)):
             raise ValueError("Unknown type: {}".format(type(value).__name__))
         return value, None
 
@@ -968,9 +971,9 @@ class CronSlices(list):
         key = value.lstrip('@').lower()
         if value.count(' ') == 4:
             return value.strip().split(' '), None
-        elif key in SPECIALS.keys():
+        if key in SPECIALS.keys():
             return SPECIALS[key].split(' '), '@' + key
-        elif value.startswith('@'):
+        if value.startswith('@'):
             raise ValueError("Unknown special '{}'".format(value))
         return [value], None
 
@@ -978,14 +981,14 @@ class CronSlices(list):
         """Return just numbered parts of this crontab"""
         return ' '.join([unicode(s) for s in self])
 
-    def render(self):
+    def render(self, specials=True):
         "Return just the first part of a cron job (the numbers or special)"
         slices = self.clean_render()
-        if self.special:
+        if self.special and specials is not False:
             if self.special == '@reboot' or \
                     SPECIALS[self.special.strip('@')] == slices:
                 return self.special
-        if not SYSTEMV:
+        if not SYSTEMV and specials is True:
             for (name, value) in SPECIALS.items():
                 if value == slices and name not in SPECIAL_IGNORE:
                     return "@%s" % name
@@ -1013,7 +1016,7 @@ class CronSlices(list):
         for month in self[3]:
             for day in self[2]:
                 try:
-                    if date(year, month, day).weekday() in weekdays:
+                    if (date(year, month, day).weekday() + 1) % 7 in weekdays:
                         result += 1
                 except ValueError:
                     continue
@@ -1036,9 +1039,8 @@ class CronSlices(list):
 
 class SundayError(KeyError):
     """Sunday was specified as 7 instead of 0"""
-    pass
 
-class Also(object):
+class Also:
     """Link range values together (appending instead of replacing)"""
     def __init__(self, obj):
         self.obj = obj
@@ -1055,7 +1057,7 @@ class Also(object):
         """Also during these"""
         return self.obj.during(*a, also=True)
 
-class CronSlice(object):
+class CronSlice:
     """Cron slice object which shows a time pattern"""
     def __init__(self, info, value=None):
         if isinstance(info, int):
@@ -1081,7 +1083,7 @@ class CronSlice(object):
                     continue
                 self.parts.append(self.parse_value(part, sunday=0))
 
-    def render(self, resolve=False):
+    def render(self, resolve=False, specials=True):
         """Return the slice rendered as a crontab.
 
         resolve - return integer values instead of enums (default False)
@@ -1187,14 +1189,14 @@ def get_cronvalue(value, enums):
     """Returns a value as int (pass-through) or a special enum value"""
     if isinstance(value, int):
         return value
-    elif unicode(value).isdigit():
+    if unicode(value).isdigit():
         return int(str(value))
     if not enums:
         raise KeyError("No enumeration allowed")
     return CronValue(unicode(value), enums)
 
 
-class CronValue(object): # pylint: disable=too-few-public-methods
+class CronValue: # pylint: disable=too-few-public-methods
     """Represent a special value in the cron line"""
     def __init__(self, value, enums):
         self.text = value
@@ -1229,7 +1231,7 @@ def _render(value, resolve=False):
     return unicode(u'{:02d}'.format(value) if ZERO_PAD else value)
 
 
-class CronRange(object):
+class CronRange:
     """A range between one value and another for a time range."""
     def __init__(self, vslice, *vrange):
         # holds an extra dangling entry, for example sundays.
@@ -1360,7 +1362,7 @@ class OrderedVariableList(OrderedDict):
         previous = self.previous
         if key in self:
             return super(OrderedVariableList, self).__getitem__(key)
-        elif previous is not None:
+        if previous is not None:
             return previous.all()[key]
         raise KeyError("Environment Variable '%s' not found." % key)
 
