@@ -199,10 +199,10 @@ class InteractionTestCase(LoggingMixin, unittest.TestCase):
         job.minute.every(2)
         job.valid = False
         with self.assertRaises(ValueError):
-            unicode(job)
+            str(job)
         # Disabled jobs still work
         job.enabled = False
-        unicode(job)
+        str(job)
 
         job.enabled = True
         with self.assertRaises(ValueError):
@@ -210,6 +210,11 @@ class InteractionTestCase(LoggingMixin, unittest.TestCase):
         self.crontab.render(errors=False)
         self.assertFalse(job.enabled)
 
+        job = self.crontab.new()
+        self.assertFalse(job.is_valid())
+        job.set_command("now_valid")
+        self.assertTrue(job.is_valid())
+
     def test_15_slices(self):
         """Invalid Slices"""
         mon = CronSlices('* * * * *')
