@@ -129,8 +129,8 @@ class RangeTestCase(LoggingMixin, unittest.TestCase):
     def test_06_backwards(self):
         """Test backwards ranges for error"""
         tab = CronTab(tab="* * * * 3-1 command")
-        self.assertEqual(str(tab), "* * * * 1-3 command\n")
-        self.assertLog("warning", "Bad range '3-1'")
+        self.assertEqual(str(tab), "# DISABLED LINE\n# * * * * 3-1 command\n")
+        self.assertLog("error", "Bad range '3-1'")
 
 if __name__ == '__main__':
     test_support.run_unittest(RangeTestCase)
