@@ -42,7 +42,7 @@ class DummyStdout(object):
 
 BASIC = '@hourly firstcommand\n\n'
 USER = '\n*/4 * * * * user_command # user_comment\n\n\n'
-crontab.CRONCMD = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))
+crontab.CRON_COMMAND = "%s %s" % (sys.executable, os.path.join(TEST_DIR, 'data', 'crontest'))
 
 def flush():
     pass
@@ -145,7 +145,7 @@ class UseTestCase(unittest.TestCase):
         self.assertTrue(item.is_enabled())
         with self.assertRaises(UnboundLocalError):
             item.delete()
-        item.command = str('nothing')
+        item.set_command('nothing')
         self.assertEqual(item.render(), '* * * * * nothing')
 
     def test_10_time_object(self):
@@ -198,10 +198,20 @@ class UseTestCase(unittest.TestCase):
         self.assertFalse(CronSlices.is_valid('* * * * FRO-TOO'))
         self.assertFalse(CronSlices.is_valid('@retool'))
 
+    def test_21_slice_special(self):
+        """Rendering can be done without specials"""
+        cronitem = crontab.CronItem('true')
+        cronitem.setall('0 0 * * *')
+        self.assertEqual(cronitem.render(specials=True), '@daily true')
+        self.assertEqual(cronitem.render(specials=None), '0 0 * * * true')
+        cronitem.setall('@daily')
+        self.assertEqual(cronitem.render(specials=None), '@daily true')
+        self.assertEqual(cronitem.render(specials=False), '0 0 * * * true')
+
     def test_25_open_pipe(self):
         """Test opening pipes"""
-        from crontab import open_pipe, CRONCMD
-        pipe = open_pipe(CRONCMD, h=None, a='one', abc='two')
+        from crontab import open_pipe, CRON_COMMAND
+        pipe = open_pipe(CRON_COMMAND, h=None, a='one', abc='two')
         (out, err) = pipe.communicate()
         self.assertEqual(err, b'')
         self.assertEqual(out, b'--abc=two|-a|-h|one\n')
